package com.cim.idm.utils;

import com.cim.idm.framework.IDMFrameServiceProxy;
import com.cim.idm.wmsextend.generic.errorHandler.CustomException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;

@Component
public class CodeGenerator {

	private static Log log = LogFactory.getLog(CodeGenerator.class);

	private static final String DATE_FORMAT = "yyyyMMdd";
	private static final String PREFIX = "PT";
	public static String generateCode() throws CustomException {


		String currentDateStr = new SimpleDateFormat(DATE_FORMAT).format(new Date());
		String newPalletNo = null;
		boolean success = false;

		while (!success) {

			String maxSeqNoQuery = "SELECT NVL(MAX(TO_NUMBER(SUBSTR(PALLETNO, " +
					"(LENGTH('" + PREFIX + "') + LENGTH(:dateStr) + 1), 3))), 0) + 1 AS new_seq " +
					"FROM MATERIALPACKINGPALLET " +
					"WHERE SUBSTR(PALLETNO, 1, LENGTH('" + PREFIX + "') + LENGTH(:dateStr)) = '" + PREFIX + "' || :dateStr";

			try {
				log.info("maxSeqNoQuery: " + maxSeqNoQuery);
				Map<String, String> bindMap = new HashMap<String, String>();
				bindMap.put("dateStr", currentDateStr);
				Map<String, Object> maxSeqNoResult = IDMFrameServiceProxy.getSqlTemplate().queryForMap(maxSeqNoQuery,bindMap);

				int newSeqNo = 0;
				if (maxSeqNoResult != null && maxSeqNoResult.containsKey("new_seq")) {
					newSeqNo = Integer.parseInt(maxSeqNoResult.get("new_seq").toString());
				}
				// 确保流水码不超过999
				if (newSeqNo > 999) {
					throw new CustomException("流水码超过了数字999");
				}
				String newSeqNoStr = String.format("%03d", newSeqNo);
				newPalletNo = PREFIX + currentDateStr + newSeqNoStr;

				success = true;
			}catch (Exception e) {
				log.error("Failed to generate pallet number: " + e.getMessage());
				throw new CustomException("生成流水码错误: " + e.getMessage());
			}

		}
		return newPalletNo;
	}

	// 修改数量，条码里批次拆分获取流水码
	public static String packingGenerateCode(String materialpackingname,String qty2) throws CustomException {

		// 3. 遍历结果集，生成新条码并更新数据库
		List<Integer> idsToUpdate = new ArrayList<Integer>();
		List<String> newNames = new ArrayList<String>();
		int sequence = 1; // 假设从1开始计数，实际中可能需要从数据库获取当前最大值或使用其他逻辑

		String[] parts = materialpackingname.split("\\|");
		if (parts.length < 7) {
			throw new IllegalArgumentException("Invalid barcode format: " + materialpackingname);
		}

		// 构建新的条码部分，假设在第五部分（索引4，因为索引从0开始）后添加流水码
		StringBuilder newBasePart = new StringBuilder();
		for (int i = 0; i < parts.length - 2; i++) { // 不包括最后两个部分
			newBasePart.append(parts[i]).append("|");
		}
		newBasePart.append(parts[4]).append("-").append(sequence++); // 在第五部分后添加流水码
//		for (int i = 6; i < parts.length; i++) { // 添加最后两个部分
//			newBasePart.append("|").append(parts[i]);
//		}
		// 添加倒数第二个部分（假设是日期）
		newBasePart.append("|").append(parts[5]);
		// 添加新的数量部分
		newBasePart.append("|").append(qty2); // 使用传入的新数量值

//		idsToUpdate.add(id);
		newNames.add(newBasePart.toString());

		return newBasePart.toString();
	}
}
